@@ -22,6 +22,7 @@ typedef enum transfer_function_t
   ccl_boltzmann_camb    = 4,
   ccl_boltzmann_isitgr  = 5,
   ccl_pklin_from_input  = 6,
+  ccl_eisenstein_hu_nowiggles = 7,
 } transfer_function_t;
 
 /**
@@ -33,12 +34,13 @@ typedef enum transfer_function_t
  */
 typedef enum matter_power_spectrum_t
 {
-    ccl_pknl_none        = 0,
-    ccl_linear           = 1,
-    ccl_halofit          = 2,
-    ccl_halo_model       = 3,
-    ccl_emu              = 4,
-    ccl_pknl_from_input  = 5
+    ccl_pknl_none          = 0,
+    ccl_linear             = 1,
+    ccl_halofit            = 2,
+    ccl_halo_model         = 3,
+    ccl_emu                = 4,
+    ccl_pknl_from_input    = 5,
+    ccl_pknl_from_boltzman = 6
 } matter_power_spectrum_t;
 
 /**
