@@ -190,6 +190,9 @@ typedef struct ccl_gsl_params {
   size_t HM_LIMIT;
   int HM_INT_METHOD;
 
+  // Flags for using spline integration
+  bool NZ_NORM_SPLINE_INTEGRATION;
+  bool LENSING_KERNEL_SPLINE_INTEGRATION;
 } ccl_gsl_params;
 
 extern const ccl_gsl_params default_gsl_params;
@@ -348,24 +351,6 @@ ccl_parameters ccl_parameters_create(double Omega_c, double Omega_b, double Omeg
  */
 void ccl_parameters_free(ccl_parameters * params);
 
-
-/**
- * Write a cosmology parameters object to a file in yaml format, .
- * @param params Cosmological parameters
- * @param filename Name of file to create and write
- * @param status Status flag. 0 if there are no errors, nonzero otherwise.
- * @return void
- */
-void ccl_parameters_write_yaml(ccl_parameters * params, const char * filename, int * status);
-
-/**
- * Read a cosmology parameters object from a file in yaml format, .
- * @param filename Name of existing file to read from
- * @param status Status flag. 0 if there are no errors, nonzero otherwise.
- * @return cosmo Cosmological parameters
- */
-ccl_parameters ccl_parameters_read_yaml(const char * filename, int *status);
-
 /**
  * Free a cosmology struct
  * @param cosmo Cosmological parameters
