@@ -258,7 +258,7 @@ def rho_x(cosmo, a, species, is_comoving=False):
 
     Returns:
         rho_x (float or array_like): Physical density of a given species
-        at a scale factor.
+        at a scale factor, in units of Msun / Mpc^3.
     """
 
     if species not in species_types.keys():
@@ -269,3 +269,38 @@ def rho_x(cosmo, a, species, is_comoving=False):
     return _vectorize_fn4(
         lib.rho_x, lib.rho_x_vec, cosmo, a,
         species_types[species], int(is_comoving))
+
+
+def sigma_critical(cosmo, a_lens, a_source):
+    """Returns the critical surface mass density.
+
+    .. math::
+         \\Sigma_{\\mathrm{crit}} = \\frac{c^2}{4\\pi G}
+          \\frac{D_{\\rm{s}}}{D_{\\rm{l}}D_{\\rm{ls}}},
+           where :math:`c` is the speed of light, :math:`G` is the
+           gravitational constant, and :math:`D_i` is the angular diameter
+           distance. The labels :math:`i =` s, l and ls denotes the distances
+           to the source, lens, and between source and lens, respectively.
+
+    Args:
+        cosmo (:class:`~pyccl.core.Cosmology`): A Cosmology object.
+        a_lens (float): lens' scale factor.
+        a_source (float or array_like): source's scale factor.
+
+    Returns:
+        float or array_like: :math:`\\Sigma_{\\mathrm{crit}}` in units
+        of :math:`\\M_{\\odot}/Mpc^2`
+    """
+    physical_constants = lib.cvar.constants
+    Ds = angular_diameter_distance(cosmo, a_source, a2=None)
+    Dl = angular_diameter_distance(cosmo, a_lens, a2=None)
+    Dls = angular_diameter_distance(cosmo, a_lens, a_source)
+    A = (
+        physical_constants.CLIGHT**2
+        * physical_constants.MPC_TO_METER
+        / (4.0 * np.pi * physical_constants.GNEWT
+           * physical_constants.SOLAR_MASS)
+    )
+
+    Sigma_crit = A * Ds / (Dl * Dls)
+    return Sigma_crit
