@@ -24,17 +24,20 @@ from .pk2d import Pk2D
 from .errors import CCLError
 
 
-def get_camb_pk_lin(cosmo):
+def get_camb_pk_lin(cosmo, nonlin=False):
     """Run CAMB and return the linear power spectrum.
 
     Args:
         cosmo (:class:`~pyccl.core.Cosmology`): Cosmological
             parameters. The cosmological parameters with
             which to run CAMB.
+        nonlin (:obj:`bool`, optional): Whether to compute and return the
+            non-linear power spectrum as well.
 
     Returns:
-        :class:`~pyccl.pk2d.Pk2D`: Power spectrum \
-            object. The linear power spectrum.
+        :class:`~pyccl.pk2d.Pk2D`: Power spectrum object. The linear power \
+            spectrum. If ``nonlin=True``, returns a tuple \
+            ``(pk_lin, pk_nonlin)``.
     """
 
     # Comment from Jarvis: TODO clean up this and other assert
@@ -43,11 +46,18 @@ def get_camb_pk_lin(cosmo):
         "You must have the `camb` python package "
         "installed to run CCL with CAMB!")
 
+    # Get extra CAMB parameters that were specified
+    extra_camb_params = {}
+    try:
+        extra_camb_params = cosmo["extra_parameters"]["camb"]
+    except (KeyError, TypeError):
+        pass
+
     # z sampling from CCL parameters
     na = lib.get_pk_spline_na(cosmo.cosmo)
     status = 0
     a_arr, status = lib.get_pk_spline_a(cosmo.cosmo, na, status)
-    check(status)
+    check(status, cosmo=cosmo)
     a_arr = np.sort(a_arr)
     zs = 1.0 / a_arr - 1
     zs = np.clip(zs, 0, np.inf)
@@ -129,21 +139,43 @@ def get_camb_pk_lin(cosmo):
         cp.ombh2 * (camb.constants.COBE_CMBTemp / cp.TCMB) ** 3,
         delta_neff)
 
+    camb_de_models = ['DarkEnergyPPF', 'ppf', 'DarkEnergyFluid', 'fluid']
+    camb_de_model = extra_camb_params.get('dark_energy_model', 'fluid')
+    if camb_de_model not in camb_de_models:
+        raise ValueError("The only dark energy models CCL supports with"
+                         " camb are fluid and ppf.")
     cp.set_classes(
-        dark_energy_model=camb.dark_energy.DarkEnergyFluid
+        dark_energy_model=camb_de_model
     )
+
+    if camb_de_model not in camb_de_models[:2] and cosmo['wa'] and \
+            (cosmo['w0'] < -1 - 1e-6 or
+                1 + cosmo['w0'] + cosmo['wa'] < - 1e-6):
+        raise ValueError("If you want to use w crossing -1,"
+                         " then please set the dark_energy_model to ppf.")
     cp.DarkEnergy.set_params(
         w=cosmo['w0'],
         wa=cosmo['wa']
     )
-    # cp.set_cosmology()
+
+    if nonlin:
+        cp.NonLinearModel = camb.nonlinear.Halofit()
+        halofit_version = extra_camb_params.get("halofit_version", "mead")
+        options = {k: extra_camb_params[k] for k in
+                   ["HMCode_A_baryon",
+                    "HMCode_eta_baryon",
+                    "HMCode_logT_AGN"] if k in extra_camb_params}
+        cp.NonLinearModel.set_params(halofit_version=halofit_version,
+                                     **options)
+
     cp.set_matter_power(
         redshifts=[_z for _z in zs],
-        kmax=10,
-        nonlinear=False)
-    assert cp.NonLinear == camb.model.NonLinear_none
+        kmax=extra_camb_params.get("kmax", 10.0),
+        nonlinear=nonlin)
+    if not nonlin:
+        assert cp.NonLinear == camb.model.NonLinear_none
 
-    cp.set_for_lmax(5000)
+    cp.set_for_lmax(extra_camb_params.get("lmax", 5000))
     cp.InitPower.set_params(
         As=A_s_fid,
         ns=cosmo['n_s'])
@@ -178,7 +210,38 @@ def get_camb_pk_lin(cosmo):
         extrap_order_hik=2,
         cosmo=cosmo)
 
-    return pk_lin
+    if not nonlin:
+        return pk_lin
+    else:
+        k, z, pk = camb_res.get_linear_matter_power_spectrum(
+            hubble_units=True, nonlinear=True)
+
+        # convert to non-h inverse units
+        k *= cosmo['h']
+        pk /= (h2 * cosmo['h'])
+
+        # now build interpolant
+        nk = k.shape[0]
+        lk_arr = np.log(k)
+        a_arr = 1.0 / (1.0 + z)
+        na = a_arr.shape[0]
+        sinds = np.argsort(a_arr)
+        a_arr = a_arr[sinds]
+        ln_p_k_and_z = np.zeros((na, nk), dtype=np.float64)
+        for i, sind in enumerate(sinds):
+            ln_p_k_and_z[i, :] = np.log(pk[sind, :])
+
+        pk_nonlin = Pk2D(
+            pkfunc=None,
+            a_arr=a_arr,
+            lk_arr=lk_arr,
+            pk_arr=ln_p_k_and_z,
+            is_logp=True,
+            extrap_order_lok=1,
+            extrap_order_hik=2,
+            cosmo=cosmo)
+
+        return pk_lin, pk_nonlin
 
 
 def get_isitgr_pk_lin(cosmo):
@@ -204,11 +267,18 @@ def get_isitgr_pk_lin(cosmo):
             *e.args)
         raise
 
+    # Get extra CAMB parameters that were specified
+    extra_camb_params = {}
+    try:
+        extra_camb_params = cosmo["extra_parameters"]["camb"]
+    except (KeyError, TypeError):
+        pass
+
     # z sampling from CCL parameters
     na = lib.get_pk_spline_na(cosmo.cosmo)
     status = 0
     a_arr, status = lib.get_pk_spline_a(cosmo.cosmo, na, status)
-    check(status)
+    check(status, cosmo=cosmo)
     a_arr = np.sort(a_arr)
     zs = 1.0 / a_arr - 1
     zs = np.clip(zs, 0, np.inf)
@@ -266,7 +336,7 @@ def get_isitgr_pk_lin(cosmo):
 
     delta_neff = cosmo['Neff'] - 3.046  # used for BBN YHe comps
 
-    # ISiTGR built on  CAMB which defines a neutrino degeneracy
+    # ISiTGR built on CAMB which defines a neutrino degeneracy
     # factor as T_i = g^(1/4)*T_nu
     # where T_nu is the standard neutrino temperature from first order
     # computations
@@ -299,9 +369,19 @@ def get_isitgr_pk_lin(cosmo):
         cp.ombh2 * (isitgr.constants.COBE_CMBTemp / cp.TCMB) ** 3,
         delta_neff)
 
+    camb_de_models = ['DarkEnergyPPF', 'ppf', 'DarkEnergyFluid', 'fluid']
+    camb_de_model = extra_camb_params.get('dark_energy_model', 'fluid')
+    if camb_de_model not in camb_de_models:
+        raise ValueError("The only dark energy models CCL supports with"
+                         " camb are fluid and ppf.")
     cp.set_classes(
-        dark_energy_model=isitgr.dark_energy.DarkEnergyFluid
+        dark_energy_model=camb_de_model
     )
+    if camb_de_model not in camb_de_models[:2] and cosmo['wa'] and \
+            (cosmo['w0'] < -1 - 1e-6 or
+                1 + cosmo['w0'] + cosmo['wa'] < - 1e-6):
+        raise ValueError("If you want to use w crossing -1,"
+                         " then please set the dark_energy_model to ppf.")
     cp.DarkEnergy.set_params(
         w=cosmo['w0'],
         wa=cosmo['wa']
@@ -428,7 +508,7 @@ def get_class_pk_lin(cosmo):
         na = lib.get_pk_spline_na(cosmo.cosmo)
         status = 0
         a_arr, status = lib.get_pk_spline_a(cosmo.cosmo, na, status)
-        check(status)
+        check(status, cosmo=cosmo)
 
         # FIXME - getting the lowest CLASS k value from the python interface
         # appears to be broken - setting to 1e-5 which is close to the
