@@ -5,6 +5,7 @@ can compute a set of theoretical predictions.
 import warnings
 import numpy as np
 import yaml
+from inspect import getmembers, isfunction, signature
 
 from . import ccllib as lib
 from .errors import CCLError, CCLWarning
@@ -18,6 +19,7 @@ from .bcm import bcm_correct_pk2d
 transfer_function_types = {
     None: lib.transfer_none,
     'eisenstein_hu': lib.eisenstein_hu,
+    'eisenstein_hu_nowiggles': lib.eisenstein_hu_nowiggles,
     'bbks': lib.bbks,
     'boltzmann_class': lib.boltzmann_class,
     'boltzmann_camb': lib.boltzmann_camb,
@@ -31,7 +33,8 @@ matter_power_spectrum_types = {
     'halofit': lib.halofit,
     'linear': lib.linear,
     'emu': lib.emu,
-    'calculator': lib.pknl_from_input
+    'calculator': lib.pknl_from_input,
+    'camb': lib.pknl_from_boltzman
 }
 
 baryons_power_spectrum_types = {
@@ -89,7 +92,7 @@ class Cosmology(object):
               the generic relative accuracy for integration by executing
               ``c = Cosmology(...); c.cosmo.gsl_params.INTEGRATION_EPSREL \
 = 1e-5``.
-              See the module level documetaion of `pyccl.core` for details.
+              See the module level documentation of `pyccl.core` for details.
 
     Args:
         Omega_c (:obj:`float`): Cold dark matter density fraction.
@@ -164,14 +167,51 @@ class Cosmology(object):
             Defaults to 'tinker10' (2010).
         halo_concentration (:obj:`str`, optional): The halo concentration
             relation to use. Defaults to Duffy et al. (2008) 'duffy2008'.
-        emulator_neutrinos: `str`, optional): If using the emulator for
+        emulator_neutrinos (:obj:`str`, optional): If using the emulator for
             the power spectrum, specified treatment of unequal neutrinos.
             Options are 'strict', which will raise an error and quit if the
             user fails to pass either a set of three equal masses or a sum with
             m_nu_type = 'equal', and 'equalize', which will redistribute
-            masses to be equal right before calling the emualtor but results in
+            masses to be equal right before calling the emulator but results in
             internal inconsistencies. Defaults to 'strict'.
+        extra_parameters (:obj:`dict`, optional): Dictionary holding extra
+            parameters. Currently supports extra parameters for CAMB, with
+            details described below. Defaults to None.
+
+    Currently supported extra parameters for CAMB are:
+
+        * `halofit_version`
+        * `HMCode_A_baryon`
+        * `HMCode_eta_baryon`
+        * `HMCode_logT_AGN`
+        * `kmax`
+        * `lmax`
+        * `dark_energy_model`
+
+    Consult the CAMB documentation for their usage. These parameters are passed
+    in a :obj:`dict` to `extra_parameters` as::
+
+        extra_parameters = {"camb": {"halofit_version": "mead2020_feedback",
+                                     "HMCode_logT_AGN": 7.8}}
+
     """
+    # Go through all functions in the main package and the subpackages
+    # and make every function that takes `cosmo` as its first argument
+    # an attribute of this class.
+    from . import background, bcm, boltzmann, \
+        cls, correlations, covariances, neutrinos, \
+        pk2d, power, tk3d, tracers, halos, nl_pt
+    subs = [background, boltzmann, bcm, cls, correlations, covariances,
+            neutrinos, pk2d, power, tk3d, tracers, halos, nl_pt]
+    funcs = [getmembers(sub, isfunction) for sub in subs]
+    funcs = [func for sub in funcs for func in sub]
+    for name, func in funcs:
+        pars = signature(func).parameters
+        if list(pars)[0] == "cosmo":
+            vars()[name] = func
+    del background, boltzmann, bcm, cls, correlations, covariances, \
+        neutrinos, pk2d, power, tk3d, tracers, halos, nl_pt
+
     def __init__(
             self, Omega_c=None, Omega_b=None, h=None, n_s=None,
             sigma8=None, A_s=None,
@@ -185,7 +225,8 @@ class Cosmology(object):
             baryons_power_spectrum='nobaryons',
             mass_function='tinker10',
             halo_concentration='duffy2008',
-            emulator_neutrinos='strict'):
+            emulator_neutrinos='strict',
+            extra_parameters=None):
 
         # going to save these for later
         self._params_init_kwargs = dict(
@@ -195,7 +236,8 @@ class Cosmology(object):
             bcm_log10Mc=bcm_log10Mc,
             bcm_etab=bcm_etab, bcm_ks=bcm_ks, mu_0=mu_0, sigma_0=sigma_0,
             c1_mg=c1_mg, c2_mg=c2_mg, lambda_mg=lambda_mg,
-            z_mg=z_mg, df_mg=df_mg)
+            z_mg=z_mg, df_mg=df_mg,
+            extra_parameters=extra_parameters)
 
         self._config_init_kwargs = dict(
             transfer_function=transfer_function,
@@ -229,57 +271,61 @@ class Cosmology(object):
         """Write a YAML representation of the parameters to file.
 
         Args:
-            filename (:obj:`str`) Filename to write parameters to.
+            filename (:obj:`str`) Filename, file pointer, or stream to write "
+                "parameters to."
         """
-        # NOTE: we use the C yaml dump here so that the parameters
-        # dumped by this object are compatible with the C yaml load function.
-        status = 0
-        status = lib.parameters_write_yaml(self._params, filename, status)
+        def make_yaml_friendly(d):
+            for k, v in d.items():
+                if isinstance(v, np.floating):
+                    d[k] = float(v)
+                elif isinstance(v, np.integer):
+                    d[k] = int(v)
+                elif isinstance(v, np.bool):
+                    d[k] = bool(v)
+                elif isinstance(v, dict):
+                    make_yaml_friendly(v)
 
-        # Check status
-        if status != 0:
-            raise IOError("Unable to write YAML file {}".format(filename))
+        params = {**self._params_init_kwargs,
+                  **self._config_init_kwargs}
+        make_yaml_friendly(params)
+
+        if isinstance(filename, str):
+            with open(filename, "w") as fp:
+                yaml.dump(params, fp,
+                          default_flow_style=False, sort_keys=False)
+        else:
+            yaml.dump(params, filename,
+                      default_flow_style=False, sort_keys=False)
 
     @classmethod
     def read_yaml(cls, filename, **kwargs):
         """Read the parameters from a YAML file.
 
         Args:
-            filename (:obj:`str`) Filename to read parameters from.
+            filename (:obj:`str`) Filename, file pointer, or stream to read
+                parameters from.
             **kwargs (dict) Additional keywords that supersede file contents
         """
-        with open(filename, 'r') as fp:
-            params = yaml.load(fp, Loader=yaml.Loader)
+        if isinstance(filename, str):
+            with open(filename, 'r') as fp:
+                params = yaml.load(fp, Loader=yaml.Loader)
+        else:
+            params = yaml.load(filename, Loader=yaml.Loader)
 
-        # Now we assemble an init for the object since the CCL YAML has
-        # extra info we don't need and different formatting.
-        inits = dict(
-            Omega_c=params['Omega_c'],
-            Omega_b=params['Omega_b'],
-            h=params['h'],
-            n_s=params['n_s'],
-            sigma8=None if params['sigma8'] == 'nan' else params['sigma8'],
-            A_s=None if params['A_s'] == 'nan' else params['A_s'],
-            Omega_k=params['Omega_k'],
-            Neff=params['Neff'],
-            w0=params['w0'],
-            wa=params['wa'],
-            bcm_log10Mc=params['bcm_log10Mc'],
-            bcm_etab=params['bcm_etab'],
-            bcm_ks=params['bcm_ks'],
-            mu_0=params['mu_0'],
-            sigma_0=params['sigma_0'],
-            c1_mg=params['c1_mg'],
-            c2_mg=params['c2_mg'],
-            lambda_mg=params['lambda_mg'])
-        if 'z_mg' in params:
-            inits['z_mg'] = params['z_mg']
-            inits['df_mg'] = params['df_mg']
+        if "sigma8" in params and params["sigma8"] == "nan":
+            del params["sigma8"]
+        if "A_s" in params and params["A_s"] == "nan":
+            del params["A_s"]
 
-        if 'm_nu' in params:
-            inits['m_nu'] = params['m_nu']
-            inits['m_nu_type'] = 'list'
+        # Get the call signature of Cosmology (i.e., the names of
+        # all arguments)
+        init_param_names = signature(cls).parameters.keys()
 
+        # Read the values we need from the loaded yaml dictionary. Missing
+        # values take their default values from Cosmology.__init__
+        inits = {k: params[k] for k in init_param_names if k in params}
+
+        # Overwrite with extra values
         inits.update(kwargs)
 
         return cls(**inits)
@@ -364,7 +410,8 @@ class Cosmology(object):
             w0=None, wa=None, T_CMB=None,
             bcm_log10Mc=None, bcm_etab=None, bcm_ks=None,
             mu_0=None, sigma_0=None, c1_mg=None, c2_mg=None, lambda_mg=None,
-            z_mg=None, df_mg=None, Omega_g=None):
+            z_mg=None, df_mg=None, Omega_g=None,
+            extra_parameters=None):
         """Build a ccl_parameters struct"""
 
         # Check to make sure Omega_k is within reasonable bounds.
@@ -568,6 +615,8 @@ class Cosmology(object):
         try:
             if key == 'm_nu':
                 val = lib.parameters_get_nu_masses(self._params, 3)
+            elif key == 'extra_parameters':
+                val = self._params_init_kwargs["extra_parameters"]
             else:
                 val = getattr(self._params, key)
         except AttributeError:
@@ -715,7 +764,7 @@ class Cosmology(object):
 
         if (self['N_nu_mass'] > 0 and
                 self._config_init_kwargs['transfer_function'] in
-                ['bbks', 'eisenstein_hu']):
+                ['bbks', 'eisenstein_hu', 'eisenstein_hu_nowiggles']):
             warnings.warn(
                 "The '%s' linear power spectrum model does not properly "
                 "account for massive neutrinos!" %
@@ -725,7 +774,7 @@ class Cosmology(object):
         if self._config_init_kwargs['matter_power_spectrum'] == 'emu':
             warnings.warn(
                 "None of the linear power spectrum models in CCL are "
-                "consistent with that implictly used in the emulated "
+                "consistent with that implicitly used in the emulated "
                 "non-linear power spectrum!",
                 category=CCLWarning)
 
@@ -746,8 +795,28 @@ class Cosmology(object):
             rescale_mg = False
             pk = get_isitgr_pk_lin(self)
         elif trf == 'boltzmann_camb':
-            pk = get_camb_pk_lin(self)
-        elif trf in ['bbks', 'eisenstein_hu']:
+            pk_nl_from_camb = False
+            if self._config_init_kwargs['matter_power_spectrum'] == "camb":
+                pk_nl_from_camb = True
+            pk = get_camb_pk_lin(self, nonlin=pk_nl_from_camb)
+            if pk_nl_from_camb:
+                pk, pk_nl = pk
+                self._pk_nl['delta_matter:delta_matter'] = pk_nl
+                self._has_pk_nl = True
+                rescale_mg = False
+                rescale_s8 = False
+                if abs(self["mu_0"]) > 1e-14:
+                    warnings.warn("You want to compute the non-linear power "
+                                  "spectrum using CAMB. This cannot be "
+                                  "consistently done with mu_0 > 0.",
+                                  category=CCLWarning)
+                if np.isfinite(self["sigma8"]) \
+                        and not np.isfinite(self["A_s"]):
+                    raise CCLError("You want to compute the non-linear "
+                                   "power spectrum using CAMB and specified"
+                                   " sigma8 but the non-linear power spectrum "
+                                   "cannot be consistenty rescaled.")
+        elif trf in ['bbks', 'eisenstein_hu', 'eisenstein_hu_nowiggles']:
             rescale_s8 = False
             rescale_mg = False
             pk = Pk2D.pk_from_model(self,
@@ -838,6 +907,10 @@ class Cosmology(object):
         if (mps != 'emu') and (mps is not None):
             self.compute_linear_power()
 
+        if mps == "camb" and self._has_pk_nl:
+            # Already computed
+            return
+
         pk = None
         if mps is None:
             raise CCLError("You want to compute the non-linear power "
@@ -1221,6 +1294,15 @@ class CosmologyCalculator(Cosmology):
             raise ValueError("`pk_linear` must contain keys 'a', 'k' "
                              "and 'delta_matter:delta_matter' "
                              "(at least)")
+
+        # Check that `a` is a monotonically increasing array.
+        if not np.array_equal(pk_linear['a'], np.sort(pk_linear['a'])):
+            raise ValueError("Input scale factor array in `pk_linear` is not "
+                             "monotonically increasing.")
+
+        # needed for high-z extrapolation
+        self.compute_growth()
+
         na = len(pk_linear['a'])
         nk = len(pk_linear['k'])
         lk = np.log(pk_linear['k'])
@@ -1256,6 +1338,11 @@ class CosmologyCalculator(Cosmology):
         if (('a' not in pk_nonlin) or ('k' not in pk_nonlin)):
             raise ValueError("`pk_nonlin` must contain keys "
                              "'a' and 'k' (at least)")
+        # Check that `a` is a monotonically increasing array.
+        if not np.array_equal(pk_nonlin['a'], np.sort(pk_nonlin['a'])):
+            raise ValueError("Input scale factor array in `pk_nonlin` is not "
+                             "monotonically increasing.")
+
         if ((not has_nonlin_model) and
                 ('delta_matter:delta_matter' not in pk_nonlin)):
             raise ValueError("`pk_nonlin` must contain key "
