@@ -1,8 +1,8 @@
 # Halo mass definitions
 from .massdef import (  # noqa
     mass2radius_lagrangian, MassDef,
-    MassDef200m, MassDef200c,
-    MassDefVir)
+    MassDef200m, MassDef200c, MassDef500c,
+    MassDefVir, convert_concentration)
 
 # Halo concentration
 from .concentration import (  # noqa
@@ -12,6 +12,7 @@ from .concentration import (  # noqa
     ConcentrationPrada12,
     ConcentrationKlypin11,
     ConcentrationDuffy08,
+    ConcentrationIshiyama21,
     ConcentrationConstant,
     concentration_from_name)
 
@@ -50,4 +51,11 @@ from .halo_model import (  # noqa
     halomod_power_spectrum,
     halomod_Pk2D,
     halomod_trispectrum_1h,
-    halomod_Tk3D_1h)
+    halomod_Tk3D_1h,
+    halomod_Tk3D_SSC,
+    halomod_Tk3D_SSC_linear_bias)
+
+# CIB profiles
+from .profiles_cib import (  # noqa
+    HaloProfileCIBShang12,
+    Profile2ptCIB)
