@@ -142,11 +142,11 @@ class MassFunc(object):
         status = 0
         sigM, status = lib.sigM_vec(cosmo.cosmo, a, logM,
                                     len(logM), status)
-        check(status)
+        check(status, cosmo=cosmo)
         # dlogsigma(M)/dlog10(M)
         dlns_dlogM, status = lib.dlnsigM_dlogM_vec(cosmo.cosmo, a, logM,
                                                    len(logM), status)
-        check(status)
+        check(status, cosmo=cosmo)
 
         rho = (lib.cvar.constants.RHO_CRITICAL *
                cosmo['Omega_m'] * cosmo['h']**2)
@@ -257,7 +257,7 @@ class MassFuncSheth99(MassFunc):
         if self.use_delta_c_fit:
             status = 0
             delta_c, status = lib.dc_NakamuraSuto(cosmo.cosmo, a, status)
-            check(status)
+            check(status, cosmo=cosmo)
         else:
             delta_c = 1.68647
 
@@ -396,10 +396,10 @@ class MassFuncDespali16(MassFunc):
     def _get_fsigma(self, cosmo, sigM, a, lnM):
         status = 0
         delta_c, status = lib.dc_NakamuraSuto(cosmo.cosmo, a, status)
-        check(status)
+        check(status, cosmo=cosmo)
 
         Dv, status = lib.Dv_BryanNorman(cosmo.cosmo, a, status)
-        check(status)
+        check(status, cosmo=cosmo)
 
         x = np.log10(self.mdef.get_Delta(cosmo, a) *
                      omega_x(cosmo, a, self.mdef.rho_type) / Dv)
