@@ -1,6 +1,6 @@
 from .. import ccllib as lib
 from ..core import check
-from ..background import h_over_h0
+from ..background import h_over_h0, sigma_critical
 from ..power import sigmaM
 from ..pyutils import resample_array, _fftlog_transform
 from .concentration import Concentration
@@ -36,6 +36,7 @@ class HaloProfile(object):
     calculation.
     """
     name = 'default'
+    is_number_counts = False
 
     def __init__(self):
         self.precision_fftlog = {'padding_lo_fftlog': 0.1,
@@ -275,6 +276,130 @@ class HaloProfile(object):
                                                 is_cumul2d=True)
         return s_r_t
 
+    def convergence(self, cosmo, r, M, a_lens, a_source, mass_def=None):
+        """ Returns the convergence as a function of cosmology,
+        radius, halo mass and the scale factors of the source
+        and the lens.
+
+        .. math::
+           \\kappa(R) = \\frac{\\Sigma(R)}{\\Sigma_{\\mathrm{crit}}},\\
+
+        where :math:`\\Sigma(R)` is the 2D projected surface mass density.
+
+        Args:
+            cosmo (:class:`~pyccl.core.Cosmology`): A Cosmology object.
+            r (float or array_like): comoving radius in Mpc.
+            M (float or array_like): halo mass in units of M_sun.
+            a_lens (float): scale factor of lens.
+            a_source (float or array_like): scale factor of source.
+                If array_like, it must have the same shape as `r`.
+            mass_def (:class:`~pyccl.halos.massdef.MassDef`):
+                a mass definition object.
+
+        Returns:
+            float or array_like: convergence \
+                :math:`\\kappa`
+        """
+        Sigma = self.projected(cosmo, r, M, a_lens, mass_def) / a_lens**2
+        if hasattr(a_source, "__iter__"):
+            a_source = np.array(a_source)
+            a_lens = np.full_like(a_source, a_lens)
+        Sigma_crit = sigma_critical(cosmo, a_lens, a_source)
+        return Sigma / Sigma_crit
+
+    def shear(self, cosmo, r, M, a_lens, a_source, mass_def=None):
+        """ Returns the shear (tangential) as a function of cosmology,
+        radius, halo mass and the scale factors of the
+        source and the lens.
+
+        .. math::
+           \\gamma(R) = \\frac{\\Delta\\Sigma(R)}{\\Sigma_{\\mathrm{crit}}} =
+           \\frac{\\overline{\\Sigma}(< R) -
+           \\Sigma(R)}{\\Sigma_{\\mathrm{crit}}},\\
+
+        where :math:`\\overline{\\Sigma}(< R)` is the average surface density
+        within R.
+
+        Args:
+            cosmo (:class:`~pyccl.core.Cosmology`): A Cosmology object.
+            r (float or array_like): comoving radius in Mpc.
+            M (float or array_like): halo mass in units of M_sun.
+            a_lens (float): scale factor of lens.
+            a_source (float or array_like): source's scale factor.
+                If array_like, it must have the same shape as `r`.
+            mass_def (:class:`~pyccl.halos.massdef.MassDef`):
+                a mass definition object.
+
+        Returns:
+            float or array_like: shear \
+                :math:`\\gamma`
+        """
+        Sigma = self.projected(cosmo, r, M, a_lens, mass_def)
+        Sigma_bar = self.cumul2d(cosmo, r, M, a_lens, mass_def)
+        if hasattr(a_source, "__iter__"):
+            a_source = np.array(a_source)
+            a_lens = np.full_like(a_source, a_lens)
+        Sigma_crit = sigma_critical(cosmo, a_lens, a_source)
+        return (Sigma_bar - Sigma) / (Sigma_crit * a_lens**2)
+
+    def reduced_shear(self, cosmo, r, M, a_lens, a_source, mass_def=None):
+        """ Returns the reduced shear as a function of cosmology,
+        radius, halo mass and the scale factors of the
+        source and the lens.
+
+        .. math::
+           g_t (R) = \\frac{\\gamma(R)}{(1 - \\kappa(R))},\\
+
+        where :math: `\\gamma(R)` is the shear and `\\kappa(R)` is the
+                convergence.
+
+        Args:
+            cosmo (:class:`~pyccl.core.Cosmology`): A Cosmology object.
+            r (float or array_like): comoving radius in Mpc.
+            M (float or array_like): halo mass in units of M_sun.
+            a_lens (float): scale factor of lens.
+            a_source (float or array_like): source's scale factor.
+                If array_like, it must have the same shape as `r`.
+            mass_def (:class:`~pyccl.halos.massdef.MassDef`):
+                a mass definition object.
+
+        Returns:
+            float or array_like: reduced shear \
+                :math:`g_t`
+        """
+        convergence = self.convergence(cosmo, r, M, a_lens, a_source, mass_def)
+        shear = self.shear(cosmo, r, M, a_lens, a_source, mass_def)
+        return shear / (1.0 - convergence)
+
+    def magnification(self, cosmo, r, M, a_lens, a_source, mass_def=None):
+        """ Returns the magnification for input parameters.
+
+        .. math::
+           \\mu (R) = \\frac{1}{\\left[(1 - \\kappa(R))^2 -
+           \\vert \\gamma(R) \\vert^2 \\right]]},\\
+
+        where :math: `\\gamma(R)` is the shear and `\\kappa(R)` is the
+        convergence.
+
+        Args:
+            cosmo (:class:`~pyccl.core.Cosmology`): A Cosmology object.
+            r (float or array_like): comoving radius in Mpc.
+            M (float or array_like): halo mass in units of M_sun.
+            a_lens (float): scale factor of lens.
+            a_source (float or array_like): source's scale factor.
+                If array_like, it must have the same shape as `r`.
+            mass_def (:class:`~pyccl.halos.massdef.MassDef`):
+                a mass definition object.
+
+        Returns:
+            float or array_like: magnification\
+                :math:`\\mu`
+        """
+        convergence = self.convergence(cosmo, r, M, a_lens, a_source, mass_def)
+        shear = self.shear(cosmo, r, M, a_lens, a_source, mass_def)
+
+        return 1.0 / ((1.0 - convergence)**2 - np.abs(shear)**2)
+
     def _fftlog_wrap(self, cosmo, k, M, a, mass_def,
                      fourier_out=False,
                      large_padding=True):
@@ -761,7 +886,7 @@ class HaloProfileEinasto(HaloProfile):
 
         status = 0
         norm, status = lib.einasto_norm(R_s, R_M, alpha, M_use.size, status)
-        check(status)
+        check(status, cosmo=cosmo)
         norm = M_use / norm
 
         x = r_use[None, :] / R_s[:, None]
@@ -831,7 +956,7 @@ class HaloProfileHernquist(HaloProfile):
 
         status = 0
         norm, status = lib.hernquist_norm(R_s, R_M, M_use.size, status)
-        check(status)
+        check(status, cosmo=cosmo)
         norm = M_use / norm
 
         x = r_use[None, :] / R_s[:, None]
@@ -1165,8 +1290,11 @@ class HaloProfileHOD(HaloProfile):
         bmax_p (float): tilt parameter for
             :math:`\\beta_{\\rm max}`.
         a_pivot (float): pivot scale factor :math:`a_*`.
-    """
+        ns_independent (bool): drop requirement to only form
+            satellites when centrals are present.
+        """
     name = 'HOD'
+    is_number_counts = True
 
     def __init__(self, c_M_relation,
                  lMmin_0=12., lMmin_p=0., siglM_0=0.4,
@@ -1174,7 +1302,7 @@ class HaloProfileHOD(HaloProfile):
                  lM1_0=13.3, lM1_p=0., alpha_0=1.,
                  alpha_p=0., fc_0=1., fc_p=0.,
                  bg_0=1., bg_p=0., bmax_0=1., bmax_p=0.,
-                 a_pivot=1.):
+                 a_pivot=1., ns_independent=False):
         if not isinstance(c_M_relation, Concentration):
             raise TypeError("c_M_relation must be of type `Concentration`)")
 
@@ -1196,6 +1324,7 @@ class HaloProfileHOD(HaloProfile):
         self.bmax_0 = bmax_0
         self.bmax_p = bmax_p
         self.a_pivot = a_pivot
+        self.ns_independent = ns_independent
         super(HaloProfileHOD, self).__init__()
 
     def _get_cM(self, cosmo, M, a, mdef=None):
@@ -1209,7 +1338,8 @@ class HaloProfileHOD(HaloProfile):
                           fc_0=None, fc_p=None,
                           bg_0=None, bg_p=None,
                           bmax_0=None, bmax_p=None,
-                          a_pivot=None):
+                          a_pivot=None,
+                          ns_independent=None):
         """ Update any of the parameters associated with
         this profile. Any parameter set to `None` won't be updated.
 
@@ -1247,6 +1377,8 @@ class HaloProfileHOD(HaloProfile):
             bmax_p (float): tilt parameter for
                 :math:`\\beta_{\\rm max}`.
             a_pivot (float): pivot scale factor :math:`a_*`.
+            ns_independent (bool): drop requirement to only form
+                satellites when centrals are present
         """
         if lMmin_0 is not None:
             self.lMmin_0 = lMmin_0
@@ -1282,6 +1414,8 @@ class HaloProfileHOD(HaloProfile):
             self.bmax_p = bmax_p
         if a_pivot is not None:
             self.a_pivot = a_pivot
+        if ns_independent is not None:
+            self.ns_independent = ns_independent
 
     def _usat_real(self, cosmo, r, M, a, mass_def):
         r_use = np.atleast_1d(r)
@@ -1346,7 +1480,10 @@ class HaloProfileHOD(HaloProfile):
         # NFW profile
         ur = self._usat_real(cosmo, r_use, M_use, a, mass_def)
 
-        prof = Nc[:, None] * (fc + Ns[:, None] * ur)
+        if self.ns_independent:
+            prof = Nc[:, None] * fc + Ns[:, None] * ur
+        else:
+            prof = Nc[:, None] * (fc + Ns[:, None] * ur)
 
         if np.ndim(r) == 0:
             prof = np.squeeze(prof, axis=-1)
@@ -1364,7 +1501,10 @@ class HaloProfileHOD(HaloProfile):
         # NFW profile
         uk = self._usat_fourier(cosmo, k_use, M_use, a, mass_def)
 
-        prof = Nc[:, None] * (fc + Ns[:, None] * uk)
+        if self.ns_independent:
+            prof = Nc[:, None] * fc + Ns[:, None] * uk
+        else:
+            prof = Nc[:, None] * (fc + Ns[:, None] * uk)
 
         if np.ndim(k) == 0:
             prof = np.squeeze(prof, axis=-1)
@@ -1384,7 +1524,10 @@ class HaloProfileHOD(HaloProfile):
         uk = self._usat_fourier(cosmo, k_use, M_use, a, mass_def)
 
         prof = Ns[:, None] * uk
-        prof = Nc[:, None] * (2 * fc * prof + prof**2)
+        if self.ns_independent:
+            prof = 2 * Nc[:, None] * fc * prof + prof**2
+        else:
+            prof = Nc[:, None] * (2 * fc * prof + prof**2)
 
         if np.ndim(k) == 0:
             prof = np.squeeze(prof, axis=-1)
