@@ -51,11 +51,22 @@ class PTCalculator(object):
         C_window (float): `C_window` parameter used by FAST-PT
             to smooth the edges and avoid ringing. See FAST-PT
             documentation for more details.
+        k_cutoff (float): exponential cutoff scale. All power
+            spectra will be multiplied by a cutoff factor of the
+            form :math:`\\exp(-(k/k_*)^n)`, where :math:`k_*` is
+            the cutoff scale. This may be useful when using the
+            resulting power spectra to compute correlation
+            functions if some of the PT contributions do not
+            fall sufficiently fast on small scales. If `None`
+            (default), no cutoff factor will be applied.
+        n_exp_cutoff (float): exponent of the cutoff factor (see
+            `k_cutoff`).
     """
     def __init__(self, with_NC=False, with_IA=False, with_dd=True,
                  log10k_min=-4, log10k_max=2, nk_per_decade=20,
                  pad_factor=1, low_extrap=-5, high_extrap=3,
-                 P_window=None, C_window=.75):
+                 P_window=None, C_window=.75,
+                 k_cutoff=None, n_exp_cutoff=4):
         assert HAVE_FASTPT, (
             "You must have the `FAST-PT` python package "
             "installed to use CCL to get PT observables! "
@@ -76,6 +87,11 @@ class PTCalculator(object):
         nk_total = int((log10k_max - log10k_min) * nk_per_decade)
         self.ks = np.logspace(log10k_min, log10k_max, nk_total)
         n_pad = int(pad_factor * len(self.ks))
+        if k_cutoff is not None:
+            self.exp_cutoff = np.exp(-(self.ks/k_cutoff)**n_exp_cutoff)
+            self.exp_cutoff = self.exp_cutoff[:, None]
+        else:
+            self.exp_cutoff = 1
 
         self.pt = fpt.FASTPT(self.ks, to_do=to_do,
                              low_extrap=low_extrap,
@@ -114,9 +130,9 @@ class PTCalculator(object):
     def _get_dd_bias(self, pk):
         # Precompute quantities needed for number counts
         # power spectra.
-        self.dd_bias = self.pt.one_loop_dd_bias(pk,
-                                                P_window=self.P_window,
-                                                C_window=self.C_window)
+        self.dd_bias = self.pt.one_loop_dd_bias_b3nl(pk,
+                                                     P_window=self.P_window,
+                                                     C_window=self.C_window)
         self.one_loop_dd = self.dd_bias[0:1]
 
     def _get_ia_bias(self, pk):
@@ -134,7 +150,8 @@ class PTCalculator(object):
 
     def get_pgg(self, Pd1d1, g4,
                 b11, b21, bs1, b12, b22, bs2,
-                sub_lowk):
+                sub_lowk, b3nl1=None, b3nl2=None,
+                bk21=None, bk22=None, Pgrad=None):
         """ Get the number counts auto-spectrum at the internal
         set of wavenumbers (given by this object's `ks` attribute)
         and a number of redshift values.
@@ -158,6 +175,19 @@ class PTCalculator(object):
                 being correlated at the same set of input redshifts.
             sub_lowk (bool): if True, the small-scale white noise
                 contribution will be subtracted.
+            b3nl1 (array_like): 3-rd order bias for the first tracer.
+                If `None`, this contribution won't be included.
+            b3nl2 (array_like): 3-rd order bias for the second tracer.
+                If `None`, this contribution won't be included.
+            bk21 (array_like): non-local bias for the first tracer.
+                If `None`, this contribution won't be included.
+            bk22 (array_like): non-local bias for the second tracer.
+                If `None`, this contribution won't be included.
+            Pgrad (array_like): cross-correlation spectrum between
+                the matter overdensity :math:`\\delta` and its
+                Laplacian (divided by :math:`-k^2`). Evaluated at
+                the wavenumber values given by this object's `ks`
+                list. If `None`, `Pd1d1` will be used.
 
         Returns:
             array_like: 2D array of shape `(N_k, N_z)`, where `N_k` \
@@ -170,6 +200,19 @@ class PTCalculator(object):
         Pd1s2 = g4[None, :] * self.dd_bias[4][:, None]
         Pd2s2 = g4[None, :] * self.dd_bias[5][:, None]
         Ps2s2 = g4[None, :] * self.dd_bias[6][:, None]
+        Pd1p3 = g4[None, :] * self.dd_bias[8][:, None]
+        if Pgrad is None:
+            Pgrad = Pd1d1
+        Pd1k2 = Pgrad * (self.ks**2)[:, None]
+
+        if b3nl1 is None:
+            b3nl1 = np.zeros_like(g4)
+        if b3nl2 is None:
+            b3nl2 = np.zeros_like(g4)
+        if bk21 is None:
+            bk21 = np.zeros_like(g4)
+        if bk22 is None:
+            bk22 = np.zeros_like(g4)
 
         s4 = 0.
         if sub_lowk:
@@ -181,8 +224,11 @@ class PTCalculator(object):
                0.25*(b21*b22)[None, :] * (Pd2d2 - 2.*s4) +
                0.5*(b11*bs2 + b12*bs1)[None, :] * Pd1s2 +
                0.25*(b21*bs2 + b22*bs1)[None, :] * (Pd2s2 - (4./3.)*s4) +
-               0.25*(bs1*bs2)[None, :] * (Ps2s2 - (8./9.)*s4))
-        return pgg
+               0.25*(bs1*bs2)[None, :] * (Ps2s2 - (8./9.)*s4) +
+               0.5*(b12*b3nl1+b11*b3nl2)[None, :] * Pd1p3 +
+               0.5*(b12*bk21+b11*bk22)[None, :] * Pd1k2)
+
+        return pgg*self.exp_cutoff
 
     def get_pgi(self, Pd1d1, g4, b1, b2, bs, c1, c2, cd):
         """ Get the number counts - IA cross-spectrum at the
@@ -225,9 +271,10 @@ class PTCalculator(object):
         pgi = b1[None, :] * (c1[None, :] * Pd1d1 +
                              (g4*cd)[None, :] * (a00e + c00e)[:, None] +
                              (g4*c2)[None, :] * (a0e2 + b0e2)[:, None])
-        return pgi
+        return pgi*self.exp_cutoff
 
-    def get_pgm(self, Pd1d1, g4, b1, b2, bs):
+    def get_pgm(self, Pd1d1, g4, b1, b2, bs, b3nl=None,
+                bk2=None, Pgrad=None):
         """ Get the number counts - matter cross-spectrum at the
         internal set of wavenumbers (given by this object's `ks`
         attribute) and a number of redshift values.
@@ -246,6 +293,19 @@ class PTCalculator(object):
             bs (array_like): tidal bias for the number counts
                 tracer being correlated at the same set of input
                 redshifts.
+            b3nl (array_like): 3-rd order bias for the number counts
+                tracer being correlated at the same set of input
+                redshifts. If `None`, this contribution won't be
+                included.
+            bk2 (array_like): non-local bias for the number counts
+                tracer being correlated at the same set of input
+                redshifts. If `None`, this contribution won't be
+                included.
+            Pgrad (array_like): cross-correlation spectrum between
+                the matter overdensity :math:`\\delta` and its
+                Laplacian (divided by :math:`-k^2`). Evaluated at
+                the wavenumber values given by this object's `ks`
+                list. If `None`, `Pd1d1` will be used.
 
         Returns:
             array_like: 2D array of shape `(N_k, N_z)`, where `N_k` \
@@ -255,11 +315,22 @@ class PTCalculator(object):
         """
         Pd1d2 = g4[None, :] * self.dd_bias[2][:, None]
         Pd1s2 = g4[None, :] * self.dd_bias[4][:, None]
+        Pd1p3 = g4[None, :] * self.dd_bias[8][:, None]
+        if Pgrad is None:
+            Pgrad = Pd1d1
+        Pd1k2 = Pgrad*(self.ks**2)[:, None]
+        if b3nl is None:
+            b3nl = np.zeros_like(g4)
+        if bk2 is None:
+            bk2 = np.zeros_like(g4)
 
         pgm = (b1[None, :] * Pd1d1 +
                0.5 * b2[None, :] * Pd1d2 +
-               0.5 * bs[None, :] * Pd1s2)
-        return pgm
+               0.5 * bs[None, :] * Pd1s2 +
+               0.5 * b3nl[None, :] * Pd1p3 +
+               0.5 * bk2[None, :] * Pd1k2)
+
+        return pgm*self.exp_cutoff
 
     def get_pii(self, Pd1d1, g4, c11, c21, cd1,
                 c12, c22, cd2, return_bb=False,
@@ -319,9 +390,9 @@ class PTCalculator(object):
                    ((cd1*c22 + cd2*c21)*g4)[None, :] * d0ee2[:, None])
 
         if return_both:
-            return pii, pii_bb
+            return pii*self.exp_cutoff, pii_bb*self.exp_cutoff
         else:
-            return pii
+            return pii*self.exp_cutoff
 
     def get_pim(self, Pd1d1, g4, c1, c2, cd):
         """ Get the intrinsic alignment - matter cross-spectrum at
@@ -355,7 +426,7 @@ class PTCalculator(object):
         pim = (c1[None, :] * Pd1d1 +
                (g4*cd)[None, :] * (a00e + c00e)[:, None] +
                (g4*c2)[None, :] * (a0e2 + b0e2)[:, None])
-        return pim
+        return pim*self.exp_cutoff
 
     def get_pmm(self, Pd1d1_lin, g4):
         """ Get the one-loop matter power spectrum.
@@ -375,11 +446,12 @@ class PTCalculator(object):
         """
         P1loop = g4[None, :] * self.one_loop_dd[0][:, None]
         pmm = (Pd1d1_lin + P1loop)
-        return pmm
+        return pmm*self.exp_cutoff
 
 
 def get_pt_pk2d(cosmo, tracer1, tracer2=None, ptc=None,
                 sub_lowk=False, nonlin_pk_type='nonlinear',
+                nonloc_pk_type='nonlinear',
                 a_arr=None, extrap_order_lok=1, extrap_order_hik=2,
                 return_ia_bb=False, return_ia_ee_and_bb=False,
                 return_ptc=False):
@@ -409,6 +481,10 @@ def get_pt_pk2d(cosmo, tracer1, tracer2=None, ptc=None,
             to use. 'linear' for linear P(k), 'nonlinear' for the internal
             non-linear power spectrum, 'spt' for standard perturbation
             theory power spectrum. Default: 'nonlinear'.
+        nonloc_pk_type (str): type of "non-local" matter power spectrum
+            to use (i.e. the cross-spectrum between the overdensity and
+            its Laplacian divided by :math:`k^2`). Same options as
+            `nonlin_pk_type`. Default: 'nonlinear'.
         a_arr (array): an array holding values of the scale factor
             at which the power spectrum should be calculated for
             interpolation. If `None`, the internal values used by
@@ -445,7 +521,7 @@ def get_pt_pk2d(cosmo, tracer1, tracer2=None, ptc=None,
         status = 0
         na = lib.get_pk_spline_na(cosmo.cosmo)
         a_arr, status = lib.get_pk_spline_a(cosmo.cosmo, na, status)
-        check(status)
+        check(status, cosmo=cosmo)
 
     if tracer2 is None:
         tracer2 = tracer1
@@ -508,18 +584,43 @@ def get_pt_pk2d(cosmo, tracer1, tracer2=None, ptc=None,
         raise NotImplementedError("Nonlinear option %s not implemented yet" %
                                   (nonlin_pk_type))
 
+    # Compute non-local power spectrum only if needed
+    Pgrad = None
+    if (((tracer1.type == 'NC') or (tracer2.type == 'NC')) and
+            (nonloc_pk_type != nonlin_pk_type)):
+        if nonloc_pk_type == 'nonlinear':
+            Pgrad = np.array([nonlin_matter_power(cosmo, ptc.ks, a)
+                              for a in a_arr]).T
+        elif nonloc_pk_type == 'linear':
+            Pgrad = np.array([linear_matter_power(cosmo, ptc.ks, a)
+                              for a in a_arr]).T
+        elif nonloc_pk_type == 'spt':
+            pklin = np.array([linear_matter_power(cosmo, ptc.ks, a)
+                              for a in a_arr]).T
+            Pgrad = ptc.get_pmm(pklin, ga4)
+        else:
+            raise NotImplementedError("Non-local option %s "
+                                      "not implemented yet" %
+                                      (nonloc_pk_type))
+
     if (tracer1.type == 'NC'):
         b11 = tracer1.b1(z_arr)
         b21 = tracer1.b2(z_arr)
         bs1 = tracer1.bs(z_arr)
+        b31 = tracer1.b3nl(z_arr)
+        bk21 = tracer1.bk2(z_arr)
         if (tracer2.type == 'NC'):
             b12 = tracer2.b1(z_arr)
             b22 = tracer2.b2(z_arr)
             bs2 = tracer2.bs(z_arr)
+            b32 = tracer2.b3nl(z_arr)
+            bk22 = tracer2.bk2(z_arr)
 
             p_pt = ptc.get_pgg(Pd1d1, ga4,
                                b11, b21, bs1, b12, b22, bs2,
-                               sub_lowk)
+                               sub_lowk, b3nl1=b31, b3nl2=b32,
+                               bk21=bk21, bk22=bk22,
+                               Pgrad=Pgrad)
         elif (tracer2.type == 'IA'):
             c12 = tracer2.c1(z_arr)
             c22 = tracer2.c2(z_arr)
@@ -528,7 +629,8 @@ def get_pt_pk2d(cosmo, tracer1, tracer2=None, ptc=None,
                                b11, b21, bs1, c12, c22, cd2)
         elif (tracer2.type == 'M'):
             p_pt = ptc.get_pgm(Pd1d1, ga4,
-                               b11, b21, bs1)
+                               b11, b21, bs1, b3nl=b31, bk2=bk21,
+                               Pgrad=Pgrad)
         else:
             raise NotImplementedError("Combination %s-%s not implemented yet" %
                                       (tracer1.type, tracer2.type))
@@ -561,8 +663,11 @@ def get_pt_pk2d(cosmo, tracer1, tracer2=None, ptc=None,
             b12 = tracer2.b1(z_arr)
             b22 = tracer2.b2(z_arr)
             bs2 = tracer2.bs(z_arr)
+            b32 = tracer2.b3nl(z_arr)
+            bk22 = tracer2.bk2(z_arr)
             p_pt = ptc.get_pgm(Pd1d1, ga4,
-                               b12, b22, bs2)
+                               b12, b22, bs2, b3nl=b32, bk2=bk22,
+                               Pgrad=Pgrad)
         elif (tracer2.type == 'IA'):
             c12 = tracer2.c1(z_arr)
             c22 = tracer2.c2(z_arr)
