@@ -10,6 +10,40 @@ from numpy.testing import assert_raises, assert_, assert_no_warnings
 import pyccl as ccl
 
 
+def test_cosmo_methods():
+    """ Check that all pyccl functions that take cosmo
+    as their first argument are methods of the Cosmology object.
+    """
+    from inspect import getmembers, isfunction, signature
+    from pyccl import background, bcm, boltzmann, \
+        cls, correlations, covariances, neutrinos, \
+        pk2d, power, tk3d, tracers, halos, nl_pt
+    from pyccl.core import CosmologyVanillaLCDM
+    cosmo = CosmologyVanillaLCDM()
+    subs = [background, boltzmann, bcm, cls, correlations, covariances,
+            neutrinos, pk2d, power, tk3d, tracers, halos, nl_pt]
+    funcs = [getmembers(sub, isfunction) for sub in subs]
+    funcs = [func for sub in funcs for func in sub]
+    for name, func in funcs:
+        pars = signature(func).parameters
+        if list(pars)[0] == "cosmo":
+            _ = getattr(cosmo, name)
+
+    # quantitative
+    assert ccl.sigma8(cosmo) == cosmo.sigma8()
+    assert ccl.rho_x(cosmo, 1., "matter", is_comoving=False) == \
+        cosmo.rho_x(1., "matter", is_comoving=False)
+    assert ccl.get_camb_pk_lin(cosmo).eval(1., 1., cosmo) == \
+        cosmo.get_camb_pk_lin().eval(1., 1., cosmo)
+    prof = ccl.halos.HaloProfilePressureGNFW()
+    hmd = ccl.halos.MassDef200m()
+    hmf = ccl.halos.MassFuncTinker08(cosmo)
+    hbf = ccl.halos.HaloBiasTinker10(cosmo)
+    hmc = ccl.halos.HMCalculator(cosmo, massfunc=hmf, hbias=hbf, mass_def=hmd)
+    assert ccl.halos.halomod_power_spectrum(cosmo, hmc, 1., 1., prof) == \
+        cosmo.halomod_power_spectrum(hmc, 1., 1., prof)
+
+
 def test_cosmology_critical_init():
     cosmo = ccl.Cosmology(
         Omega_c=0.25,
