@@ -367,13 +367,13 @@ def test_parameters_missing():
         Omega_c=0.25, Omega_b=0.05, h=0.7, A_s=2.1e-9, n_s=0.96,
         Omega_k=None)
     assert_raises(
-            ValueError, ccl.Cosmology,
-            Omega_c=0.25, Omega_b=0.05, h=0.7, A_s=2.1e-9, n_s=0.96,
-            w0=None)
+        ValueError, ccl.Cosmology,
+        Omega_c=0.25, Omega_b=0.05, h=0.7, A_s=2.1e-9, n_s=0.96,
+        w0=None)
     assert_raises(
-            ValueError, ccl.Cosmology,
-            Omega_c=0.25, Omega_b=0.05, h=0.7, A_s=2.1e-9, n_s=0.96,
-            wa=None)
+        ValueError, ccl.Cosmology,
+        Omega_c=0.25, Omega_b=0.05, h=0.7, A_s=2.1e-9, n_s=0.96,
+        wa=None)
 
     # Check that sigma8 vs A_s is handled ok.
     assert_raises(
@@ -489,7 +489,8 @@ def test_parameters_read_write():
     with tempfile.NamedTemporaryFile(delete=False) as tmpfile:
         temp_file_name = tmpfile.name
 
-    # Write out and then eead in the parameters from that file
+    # Write out and then read in the parameters from that file
+    assert_raises(IOError, params.write_yaml, "/bogus/file/name")
     params.write_yaml(temp_file_name)
     params2 = ccl.Cosmology.read_yaml(temp_file_name)
 
@@ -498,6 +499,18 @@ def test_parameters_read_write():
     assert_almost_equal(params['Neff'], params2['Neff'])
     assert_almost_equal(params['sum_nu_masses'], params2['sum_nu_masses'])
 
+    # check overriding parameters with kwargs
+    params3 = ccl.Cosmology.read_yaml(temp_file_name,
+                                      matter_power_spectrum='emu',
+                                      n_s=1.1)
+    # check unmodified parameters are the same
+    assert_almost_equal(params['Omega_c'], params3['Omega_c'])
+    assert_almost_equal(params['Neff'], params3['Neff'])
+    # check new parameters and config correctly updated
+    assert_almost_equal(1.1, params3['n_s'])
+    assert_almost_equal(params['sum_nu_masses'], params3['sum_nu_masses'])
+    assert params3._config_init_kwargs['matter_power_spectrum'] == 'emu'
+
     # Now make a file that will be deleted so it does not exist
     # and check the right error is raise
     with tempfile.NamedTemporaryFile(delete=True) as tmpfile:
@@ -508,3 +521,9 @@ def test_parameters_read_write():
         IOError,
         params.read_yaml,
         filename=temp_file_name+"/nonexistent_directory/params.yml")
+
+
+def test_omega_k():
+    """ Check that the value of Omega_k is within reasonable bounds. """
+    assert_raises(ValueError, ccl.Cosmology, Omega_c=0.25, Omega_b=0.05, h=0.7,
+                  A_s=2.1e-9, n_s=0.96, Omega_k=-2)
