@@ -85,7 +85,9 @@ const ccl_gsl_params default_gsl_params = {
   0.0,                                 // HM_EPSABS
   1E-4,                                // HM_EPSREL
   1000,                                // HM_LIMIT
-  GSL_INTEG_GAUSS41                    // HM_INT_METHOD
+  GSL_INTEG_GAUSS41,                   // HM_INT_METHOD
+  true,                                // NZ_NORM_SPLINE_INTEGRATION
+  true                                 // LENSING_KERNEL_SPLINE_INTEGRATION
   };
 
 #undef GSL_EPSREL
@@ -284,7 +286,8 @@ ccl_cosmology * ccl_cosmology_create(ccl_parameters params, ccl_configuration co
   cosmo->computed_growth = false;
   cosmo->computed_sigma = false;
   cosmo->status = 0;
-  ccl_cosmology_set_status_message(cosmo, "");
+  // Initialise as 0-length string
+  cosmo->status_message[0] = '\0';
 
   if(cosmo->spline_params.A_SPLINE_MAX !=1.) {
     cosmo->status = CCL_ERROR_SPLINE;
@@ -476,249 +479,6 @@ ccl_parameters ccl_parameters_create(double Omega_c, double Omega_b, double Omeg
 }
 
 
-/**
- * Write a cosmology parameters object to a file in yaml format.
- * @param cosmo Cosmological parameters
- * @param f FILE* pointer opened for reading
- * @return void
- */
-void ccl_parameters_write_yaml(ccl_parameters * params, const char * filename, int *status)
-{
-  FILE * f = fopen(filename, "w");
-
-  if (!f){
-    *status = CCL_ERROR_FILE_WRITE;
-    return;
-  }
-
-#define WRITE_DOUBLE(name) fprintf(f, #name ": %le\n",params->name)
-#define WRITE_INT(name) fprintf(f, #name ": %d\n",params->name)
-
-  // Densities: CDM, baryons, total matter, curvature
-  WRITE_DOUBLE(Omega_c);
-  WRITE_DOUBLE(Omega_b);
-  WRITE_DOUBLE(Omega_m);
-  WRITE_DOUBLE(Omega_k);
-  WRITE_INT(k_sign);
-
-  // Dark Energy
-  WRITE_DOUBLE(w0);
-  WRITE_DOUBLE(wa);
-
-  // Hubble parameters
-  WRITE_DOUBLE(H0);
-  WRITE_DOUBLE(h);
-
-  // Neutrino properties
-  WRITE_DOUBLE(Neff);
-  WRITE_INT(N_nu_mass);
-  WRITE_DOUBLE(N_nu_rel);
-
-  if (params->N_nu_mass>0){
-    fprintf(f, "m_nu: [");
-    for (int i=0; i<params->N_nu_mass; i++){
-      fprintf(f, "%le, ", params->m_nu[i]);
-    }
-    fprintf(f, "]\n");
-  }
-
-  WRITE_DOUBLE(sum_nu_masses);
-  WRITE_DOUBLE(Omega_nu_mass);
-  WRITE_DOUBLE(Omega_nu_rel);
-
-  // Primordial power spectra
-  WRITE_DOUBLE(A_s);
-  WRITE_DOUBLE(n_s);
-
-  // Radiation parameters
-  WRITE_DOUBLE(Omega_g);
-  WRITE_DOUBLE(T_CMB);
-
-  // BCM baryonic model parameters
-  WRITE_DOUBLE(bcm_log10Mc);
-  WRITE_DOUBLE(bcm_etab);
-  WRITE_DOUBLE(bcm_ks);
-
-  // Modified gravity parameters
-  WRITE_DOUBLE(mu_0);
-  WRITE_DOUBLE(sigma_0);
-  WRITE_DOUBLE(c1_mg);
-  WRITE_DOUBLE(c2_mg);
-  WRITE_DOUBLE(lambda_mg);
-
-  // Derived parameters
-  WRITE_DOUBLE(sigma8);
-  WRITE_DOUBLE(Omega_l);
-  WRITE_DOUBLE(z_star);
-
-  WRITE_INT(has_mgrowth);
-  WRITE_INT(nz_mgrowth);
-
-  if (params->has_mgrowth){
-    fprintf(f, "z_mgrowth: [");
-    for (int i=0; i<params->nz_mgrowth; i++){
-      fprintf(f, "%le, ", params->z_mgrowth[i]);
-    }
-    fprintf(f, "]\n");
-
-    fprintf(f, "df_mgrowth: [");
-    for (int i=0; i<params->nz_mgrowth; i++){
-      fprintf(f, "%le, ", params->df_mgrowth[i]);
-    }
-    fprintf(f, "]\n");
-  }
-
-#undef WRITE_DOUBLE
-#undef WRITE_INT
-
-  fclose(f);
-}
-
-/**
- * Write a cosmology parameters object to a file in yaml format.
- * @param cosmo Cosmological parameters
- * @param f FILE* pointer opened for reading
- * @return void
- */
-ccl_parameters ccl_parameters_read_yaml(const char * filename, int *status) {
-  FILE * f = fopen(filename, "r");
-
-  if (!f) {
-    *status = CCL_ERROR_FILE_READ;
-    ccl_parameters bad_params;
-    ccl_raise_warning(CCL_ERROR_FILE_READ,
-                      "ccl_core.c: ccl_parameters_read_yaml(): "
-                      "Failed to read parameters from file.");
-
-    return bad_params;
-  }
-
-#define READ_DOUBLE(name) double name; *status |= (0==fscanf(f, #name ": %le\n",&name));
-#define READ_INT(name) int name; *status |= (0==fscanf(f, #name ": %d\n",&name))
-
-  // Densities: CDM, baryons, total matter, curvature
-  READ_DOUBLE(Omega_c);
-  READ_DOUBLE(Omega_b);
-  READ_DOUBLE(Omega_m);
-  READ_DOUBLE(Omega_k);
-  READ_INT(k_sign);
-
-  // Dark Energy
-  READ_DOUBLE(w0);
-  READ_DOUBLE(wa);
-
-  // Hubble parameters
-  READ_DOUBLE(H0);
-  READ_DOUBLE(h);
-
-  // Neutrino properties
-  READ_DOUBLE(Neff);
-  READ_INT(N_nu_mass);
-  READ_DOUBLE(N_nu_rel);
-
-  double mnu[3] = {0.0, 0.0, 0.0};
-  if (N_nu_mass>0){
-    *status |= (0==fscanf(f, "m_nu: ["));
-    for (int i=0; i<N_nu_mass; i++){
-      *status |= (0==fscanf(f, "%le, ", mnu+i));
-    }
-    *status |= (0==fscanf(f, "]\n"));
-  }
-
-  READ_DOUBLE(sum_nu_masses);
-  READ_DOUBLE(Omega_nu_mass);
-  READ_DOUBLE(Omega_nu_rel);
-
-  // Primordial power spectra
-  READ_DOUBLE(A_s);
-  READ_DOUBLE(n_s);
-
-  // Radiation parameters
-  READ_DOUBLE(Omega_g);
-  READ_DOUBLE(T_CMB);
-
-  // BCM baryonic model parameters
-  READ_DOUBLE(bcm_log10Mc);
-  READ_DOUBLE(bcm_etab);
-  READ_DOUBLE(bcm_ks);
-
-  // Modified gravity parameters
-  READ_DOUBLE(mu_0);
-  READ_DOUBLE(sigma_0);
-  READ_DOUBLE(c1_mg);
-  READ_DOUBLE(c2_mg);
-  READ_DOUBLE(lambda_mg);
-
-  // Derived parameters
-  READ_DOUBLE(sigma8);
-  READ_DOUBLE(Omega_l);
-  READ_DOUBLE(z_star);
-
-  READ_INT(has_mgrowth);
-  READ_INT(nz_mgrowth);
-
-  double *z_mgrowth;
-  double *df_mgrowth;
-
-
-  if (has_mgrowth){
-    z_mgrowth = malloc(nz_mgrowth*sizeof(double));
-    df_mgrowth = malloc(nz_mgrowth*sizeof(double));
-    *status |= (0==fscanf(f, "z_mgrowth: ["));
-    for (int i=0; i<nz_mgrowth; i++){
-      *status |= (0==fscanf(f, "%le, ", z_mgrowth+i));
-    }
-    *status |= (0==fscanf(f, "]\n"));
-
-    *status |= (0==fscanf(f, "df_mgrowth: ["));
-    for (int i=0; i<nz_mgrowth; i++){
-      *status |= (0==fscanf(f, "%le, ", df_mgrowth+i));
-    }
-    *status |= (0==fscanf(f, "]\n"));
-  }
-  else{
-    z_mgrowth = NULL;
-    df_mgrowth = NULL;
-  }
-
-#undef READ_DOUBLE
-#undef READ_INT
-
-  fclose(f);
-
-  if (*status) {
-    ccl_raise_warning(
-      *status,
-      "ccl_core.c: ccl_parameters_read_yaml():"
-      "Structure of YAML file incorrect: %s",
-      filename);
-  }
-
-  double norm_pk;
-
-  if (isnan(A_s)){
-    norm_pk = sigma8;
-  }
-  else{
-   norm_pk = A_s;
-  }
-
-  ccl_parameters params = ccl_parameters_create(
-    Omega_c, Omega_b, Omega_k,
-    Neff, mnu, N_nu_mass,
-    w0, wa, h, norm_pk,
-    n_s, bcm_log10Mc, bcm_etab,
-    bcm_ks, mu_0, sigma_0, c1_mg, c2_mg, lambda_mg, nz_mgrowth, z_mgrowth,
-    df_mgrowth, status);
-
-  if(z_mgrowth) free(z_mgrowth);
-  if (df_mgrowth) free(df_mgrowth);
-
-  return params;
-}
-
-
-
 /* ------- ROUTINE: ccl_data_free --------
 INPUT: ccl_data
 TASK: free the input data
@@ -751,6 +511,10 @@ void ccl_cosmology_set_status_message(ccl_cosmology * cosmo, const char * messag
 
   #pragma omp critical
   {
+    if(strlen(cosmo->status_message) != 0) {
+      ccl_raise_warning(CCL_ERROR_OVERWRITE, "Status message being overwritten:");
+      fprintf(stderr, "STATUS: %d. %s\n", cosmo->status, cosmo->status_message);
+    }
     vsnprintf(cosmo->status_message, trunc, message, va);
 
     /* if truncation happens, message[trunc - 1] is not NULL, ... will show up. */
